/*
 * @Author: zuley
 * @Date: 2021-06-01 17:28:44
 * @LastEditors: zuley
 * @LastEditTime: 2021-06-09 13:43:02
 */

interface EnvPipeOpt<T> {
  /** 环境名称 */
  name: string;
  /** 环境判断函数 */
  in: () => boolean;
  /**
   * 匹配成功的触发函数
   */
  complete?: (res?: T) => Promise<T>;
}

interface EnvNameDict {
  /** 主环境名 */
  env: string;
  /** 子环境名 */
  child: string | null;
}

export default class EnvPipe<T> {
  // 环境处理管道列表
  private envList: {
    [prop: string]: EnvPipeOpt<T>;
  } = {}
  // 启动配置
  private config: string[] = []
  /** 当前环境 */
  public actEnvName = 'default'
  // 默认函数
  public defaultPipe (): Promise<T> {
    console.log('没有设置默认管道方法')
    return null as any
  }
  // 推入函数
  public push (env: EnvPipeOpt<T>) {
    if (this.envList[env.name]) {
      console.log(`${env.name} 环境重复，请检查 PUSH 的参数`)
      return
    }
    if (!env.name.includes(":") && !env.complete) {
      console.log(`${env.name} 基座环境缺少 complete `)
      return
    }
    this.envList[env.name] = env
  }
  // 启动函数
  public async start (config: string[]): Promise<T> {
    // return new Promise(res => {
    this.config = config
    // 获取当前基座环境管道
    const actMainPipe = this.getMainPipe()
    // 如果匹配不了则执行默认管道
    if (!actMainPipe) {
      this.actEnvName = "default"
      return await this.defaultPipe()
    }
    this.actEnvName = actMainPipe.name
    const actMainData = await actMainPipe.complete!()
    const actChildPipe = this.getChildPipe(actMainPipe.name)
    // 没有子管道则直接执行管道
    if (!actChildPipe) return actMainData
    // 有子管道则执行主管道后传入子管道继续执行
    this.actEnvName = actChildPipe.name
    return actChildPipe.complete ? await actChildPipe!.complete(actMainData) : actMainData
  }
  // 获取当前基座环境管道
  private getMainPipe (): EnvPipeOpt<T> | null {
    for (const envName of this.config) {
      const env = this.envList[envName]
      if (env && env.in()) return env
    }
    return null
  }
  // 获取当前基座子管道环境
  private getChildPipe (envName: string): EnvPipeOpt<T> | null {
    return this.envList[Object.keys(this.envList).find(env => {
      return env.includes(`${envName}:`) && this.envList[env].in()
    }) || '']
  }
  /**
   * 获取环境名称字典表
   * @param envName 环境名称使用冒号 ： 隔开，前面是所处环境，后面是子环境
   * @returns 返回环境名称字典表
   */
  private getEnvNameDict (envName: string): EnvNameDict {
    const arr = envName.split(":")
    return {
      env: arr[0],
      child: arr[1] || null
    }
  }
  /**
   * 判断当前所处环境
   * @param envName 不使用冒号则只判断主环境，使用冒号则判断子环境
   * @returns 布尔值
   */
  public isEnv (envName: string): boolean {
    const EnvDict = this.getEnvNameDict(envName)
    const actEvnDict = this.getEnvNameDict(this.actEnvName)
    // 如果需要判断子环境，则直接比对 envName
    if (EnvDict.child) return envName === this.actEnvName
    // 如果不需要判断子环境，则获取字典后直接比对 env
    return EnvDict.env === actEvnDict.env
  }
  /**
   * 运行函数，匹配环境后执行对应的回调函数
   * @param envName 环境名称
   * @param fn 回调函数
   */
  public run (envName: string, fn: Function) {
    if (this.isEnv(envName)) fn()
  }
}